angular.module('Landers.lwap', [])
    .factory('Lwap', function ($state, $timeout, $rootScope, $templateRequest, $compile, $q, InjectService) {
        return {
            jo : {
                vmenu   : angular.element('#lwap-vmenu'),
                main    : angular.element('#lwap-body')
            },
            cache : {},
            launch:function(appkey) {
                $state.go('base.' + appkey);
            },
            loadTemplates:function(paths, callback){
                if (paths && !angular.isArray(paths)) paths = [paths];
                if (!paths || !paths.length) return callback();
                var promises = [];
                angular.forEach(paths, function(path){
                    var defer = $q.defer();
                    promises.push(defer.promise);
                    (function(defer){
                        $templateRequest(path).then(function(tpl){
                            defer.resolve(tpl);
                        });
                    })(defer);
                });
                $q.all(promises).then(callback);
            },
            loadAppPartial : function(key2, is_private, partial, callback) {
                var url = $rootScope.$path.apptpl(key2, is_private, partial);
                this.loadTemplates(url, callback);
            },
            getAppSourcesUrl : function(key2, is_private){
                var base = $rootScope.$path.appbase(key2, is_private);
                return [
                    base + 'style.css',
                    base + 'client.js'
                ];
            },
            injectedModules:{},
            injectModule:function(key2, inject, callback){
                var module = this.injectedModules[key2];
                if (!module) {
                    var _this = this, urls = this.getAppSourcesUrl(key2);
                    InjectService.module(urls, inject, function(module){
                        _this.injectedModules[key2] = module;
                        callback(module);
                    });
                } else {
                    callback(module);
                }
            },
            injectedApps:{},
            bootApp : function(scope, key2, callback_controller, is_private) {
                var _this = this, urls = this.getAppSourcesUrl(key2, is_private);
                if (!_this.injectedApps[key2]) {
                    InjectService.urls(urls, function(){
                        if (angular.isString(callback_controller)) {
                            _this.bootController(scope, callback_controller);
                        }
                        if (angular.isFunction(callback_controller)){
                            var base = $rootScope.$path.appbase(key2, is_private);
                            var main_url = base + 'layout.html';
                            _this.bootController(scope, main_url);
                            $timeout(callback_controller, 200);
                        }
                        _this.injectedApps[key2] = true;
                    });
                } else {
                    if (angular.isFunction(callback_controller)){
                        callback_controller && callback_controller();
                    }
                }
            },
            injectedControllers:{},
            bootController: function (scope, controller_url, reload){
                if (!this.injectedControllers[controller_url] || reload) {
                    var html = controller_url.indexOf('/') >= 0 ?
                               '<div ng-include="\'' + controller_url + '\'"></div>' :
                               '<div ng-controller="' + controller_url + '"></div>';
                    var el = $compile(html)(scope);
                    angular.element('#lwap-body').append(el);
                    if (!scope.$$phase) scope.$apply();
                    this.injectedControllers[controller_url] = true;
                };
            },
            reset : function() {
                this.injectedControllers = {};
                this.injectedApps = {};
                this.injectedModules = {};
            }
        }
    })
    .factory('LwapService', function($rootScope, BaseService){
        function LwapService(scope, key2) {
            this.__proto__ = BaseService.make(scope);
            this.apiurl = $rootScope.$path.appapi(key2);
            this.ajaxGet = function(params){
                var me = this;
                return me.ajax.get(me.apiurl, params || {});
            }
        };
        return {
            make : function(scope, key2) {
                return new LwapService(scope, key2);
            }
        };
    })
    .directive('crudDoItem', function(){
        return {
            restrict : 'A',
            link : function($scope, $element, $attrs){
                $element.addClass('doitem')
                    .children('a, button').not('.btn').addClass('btn ')
                    .each(function(){
                        if ($(this).attr('class').indexOf('btn-') < 0 ) {
                            $(this).addClass('btn-primary');
                        }
                    })
            }
        }
    })
    .factory('LwapApp', function ($rootScope, $state, $timeout, $httpParamSerializer, $templateRequest, AngularModel, InjectService, Ajax, Runtime, Dialog, Flat, Lwap) {
        var defaults = {
            dialog : {
                width : 800,
                height : '100%',
                scroll : true
            },
            partials : {
                buttons : true,
                searcher : true,
                addedit : true
            }
        };

        function LwapApp(scope, config) {

            //jQuery支持深度复制
            config = $.extend(true, {}, defaults, config);
            var that = this;
            var dialogs = [];
            var tags = [];

            //环境数据
            this.scope = scope;
            this.prop = angular.extend({}, $state.$current.data);
            if (config.prop) angular.extend(this.prop, config.prop);

            this.path = {
                base : $rootScope.$path.appbase(that.prop.key2, that.prop.is_private),
                apiurl : $rootScope.$path.appapi(that.prop.key2, that.prop.is_private),
                apibuild:function(key){
                    return this.base + 'server-' + key + '.php';
                }
            };
            tags.push(Landers.loader.css(this.path.base + 'style.css'));

            this.ajax = Ajax.make();

            this.action = {
                debug : function() {
                    that.ajax.debug();
                    return this;
                },
                loading:function(){
                    that.ajax.loading();
                    return this;
                },
                build : function(action, params){
                    var apiurl; params = params || {};
                    if (that.model) {
                        apiurl = that.model.api.buildUrl('custom', {action:action});
                    } else {
                        apiurl = that.path.apiurl;
                        params.act = action;
                    }
                    return {apiurl:apiurl, params:params};
                },
                post : function(action, data, params) {
                    var t = this.build(action, params);
                    return that.ajax.post(t.apiurl, data, t.params)
                },
                get : function(action, params) {
                    var t = this.build(action, params);
                    return that.ajax.get(t.apiurl, t.params)
                }
            };

            if ( that.prop.key1 ) {
                this.model = AngularModel.make(scope, {
                    key : that.prop.key1,
                    name : that.prop.name,
                    scope : scope,
                    url_style : 'lwap',
                    url_params : (function(){
                        var ret = config.urlParams || {};
                        if (config.parentApp) {
                            ret.appid = config.parentApp.prop.appid;
                            if ( config.parentEvent ) {
                                ret.datid = config.parentApp.listitem.data(config.parentEvent, 'id');
                            }
                        };
                        return ret;
                    })(),
                    api_url : config.api_url || that.path.apiurl,
                    templateUrls : config.templateUrls,
                    ajax:that.ajax
                    // appkey : that.prop.key2
                });
            }


            this.injectClient = function(name, inject, callback, key2){
                var path = key2 ? $rootScope.$path.appbase(key2) : that.path.base;
                var url = path + 'client-' + name + '.js';
                if (inject) {
                    InjectService.module(url, inject, callback);
                } else {
                    InjectService.module(url, callback);
                }
            };

            //扩展scope
            scope.$app =  that;
            if (config.is_active) {
                $timeout(function(){$rootScope.$activeApp = that;}, 200);
            }
            scope.$partial = function(partial, path){
                if (path === undefined) {
                    path = config.partials[partial];
                };
                if (path === undefined) {
                    path = true;
                };

                var ret;
                if ( path === true || path === false ) {
                    path = path ?
                           that.path.base + 'partials/' :
                           $rootScope.$path.root() + 'front/template/';
                    ret = path + partial + '.html';
                } else {
                    ret = $rootScope.$path.apptpl(path, partial, that.prop.is_private)
                };
                return ret;
            };

            //与内置默认$services合并
            scope.$service = angular.extend( {
                tagsInput : function(fields){
                    if (!angular.isArray(fields)) fields = [fields];
                    angular.forEach(fields, function(item){
                        var value = that.model.getData('$edit', item);
                        that.runtime.set(item, {
                            data : value ? value.toString().split(',') : [],
                            onChange : function(){
                                value = this.data.map(function(item){
                                    return item.text;
                                }).join(',');
                                that.model.setData('$edit', value, item);
                            }
                        });
                    });
                }
            }, scope.$services || {});

            if (scope.$on) {
                scope.$on('$destroy', function(){
                    angular.forEach(tags, function(tag){
                        tag.remove();
                    })

                    Lwap.reset();
                });
            }

            //初始化runtime (挂载$runtime至scope)
            this.runtime = Runtime.make(scope);
            this.runtime.set('referer', document.referrer)

            //对话框
            this.dialog = {
                _loadPartial : function(dialogConfig, callback){
                    dialogConfig = dialogConfig || {};
                    var partial = dialogConfig.partial || dialogConfig.tplkey;
                    var templateUrl = partial.indexOf('/') >= 0 ? partial : scope.$partial('dialog-' + partial);
                    $templateRequest(templateUrl).then(function(html){
                        callback(html);
                    })
                },
                show : function(dialogConfig){
                    var _this = this;
                    _this._loadPartial(dialogConfig, function(html){
                        dialogConfig.html = html;
                        delete dialogConfig.tplkey;
                        delete dialogConfig.partial;
                        Dialog.show(scope, dialogConfig);
                    })
                },
                confirm : function(dialogConfig, callbacks){
                    var _this = this;
                    _this._loadPartial(dialogConfig, function(html){
                        dialogConfig.html = html;
                        Dialog.confirm(scope, dialogConfig, callbacks);
                    })
                }
            }

            //扩展或覆盖crud中scope.$events
            scope.$events = angular.extend({} , scope.$events, {
                showEdit:function($event, item){
                    $timeout(function() {
                        config.dialog.name = that.prop.name;
                        config.dialog.partial = config.dialog.partial || 'addedit';
                        that.addedit.popup($event, item, config.dialog);
                    }, 100);
                },
                showCreate:function($event){
                    this.showEdit($event);
                },
                install : function($event) {
                    that.action.get('install')
                        .success(function(){
                            that.list.refresh();
                        })
                        .error(function(response){
                            Landers.response.msgbox(response.data);
                        })
                }
                // deleteItem : function($event, item, from_lister) {
                //     console.log(scope);
                // }
            });

            if ( that.model ) {

                this.list = that.model.list;
                this.listitem = that.model.listitem;
                // that.lister = that.crud.lister;
                this.addedit = angular.extend(that.model.addedit, {
                    showCreate : scope.$events.showCreate,
                    showEdit : scope.$events.showEdit
                });
                // this.action = that.crud.action;
                this.hooks = that.model.hooks;

            }


            this.$ = {
                _datkey : function(objectKey, subkey){
                    var datkey = ['$' + objectKey];
                    if (subkey) datkey.push(subkey);
                    return datkey.join('.');
                },
                get : function(objectKey, subkey) {
                    var datkey = this._datkey(objectKey, subkey);
                    return Flat.get(scope, datkey);
                },
                set : function(objectKey, data, subkey) {
                    var datkey = this._datkey(objectKey, subkey);
                    var ret = Flat.set(scope, datkey, data);
                    if (!scope.$$phase) scope.$apply();
                    return ret;
                },
                extend : function(objectKey, data) {
                    var datkey = this._datkey(objectKey);
                    var ret = Flat.extend(scope, data, datkey);
                    if (!scope.$$phase) scope.$apply();
                    return ret;
                }
            };

            //注册默认钩子
            // that.hooks.registe('list.before', [
            // ]);
            // that.hooks.registe('list.after', [
            // ]);
            // that.hooks.registe('edit.onload.before', [
            // ])
            // that.hooks.registe('edit.onload.after', [
            // ]);
            // that.hooks.registe('edit.submit.before', [
            // ])
            // that.hooks.registe('edit.submit.after', [
            // ]);

            scope.$enums = angular.extend({}, $rootScope.$enums || {});
            this.enums = {
                __proto__ : Flat.make(scope, scope.$enums)
            };
        };

        return {
            make:function(scope, config){
                return new LwapApp(scope, config);
            }
        }
    });