;angular.module('Landers.angular')
    .provider('AngularModel', function() {
        var provider = this;

        this.$get = ['$templateRequest', 'List', 'Flat', 'AngularApi', 'Helpers', '$rootScope', function ($templateRequest, List, Flat, AngularApi, Helpers, $rootScope) {
            var is_api_result = function(rspn) {
                    if ( !angular.isObject(rspn)) return false;
                    return  'success' in rspn &&
                            'message' in rspn &&
                            'data' in rspn;
            };

            function Model(scope, config) { // key, name, api_url, url_style, url_params, [ajax, lister, vice]

                var that = this, watchers = [];

                //定义模型标识
                if (!config.key) throw ('未定义key字段');

                //将当前模型数据容器
                var modelkey = '$' + config.key;
                this.modelkey = modelkey;

                scope[modelkey] = {
                    '$name' : config.name,
                    '$list' : {
                        'data' : null,
                        'page' : null
                    },
                    '$create' : null,
                    '$detail' : null,
                    '$items' : {
                        selected:[],
                        selectedRows:[],
                    },
                    '$edit' : {},
                    '$form' : {},
                    '$update' : null,
                    '$modify' : null,
                    '$search' : null,
                    '$errors' : {
                        '$list' : null,
                        '$edit' : null,
                        '$form' : null,
                    },
                    '$runtime' : {},
                    '$event': {},
                    '$enums': {}
                };

                this.trigger = function(){
                    var fun, event, args = [];
                    for (var i=0; i<arguments.length; i++ ) {
                        if ( i== 0) {
                            event = arguments[i];
                        } else {
                            args.push(arguments[i]);
                        }
                    }
                    var fun = scope[modelkey].$event[event];

                    if ( fun ) {
                        fun.apply(scope[modelkey].$event, args)
                    }
                };

                this.getDataKey = function( key, subkey ) {
                    return modelkey + '.' + key + (subkey ? '.' + subkey : '');
                };

                this.extendData = function(key, extdata, subkey) {
                    var data = that.getData(key, subkey);
                    Flat.extend(data, extdata);
                    if (!scope.$$phase) scope.$apply();
                    return this;
                };

                this.setData = function(key, data, subkey) {
                    var datakey = modelkey + '.' + key;
                    Flat.set(scope, datakey, data, subkey);
                    if (!scope.$$phase) scope.$apply();
                    return this;
                };

                this.setDataToString = function(keys, subkey) {
                    if ( !angular.isArray(keys) ) keys = [keys];
                    keys.map(function(key){
                        var value = that.getData(key, subkey);
                        that.setData(key, value.toString(), subkey);
                    })
                };

                this.getData = function(key, subkey) {
                    var datakey = modelkey + '.' + key;
                    return Flat.get(scope, datakey, subkey);
                };

                this.watchers = [];
                this.watch = function(object, callback) {
                    if (angular.isString(object)) {
                        object = modelkey + '.' + object;
                    }
                    that.watchers.push(scope.$watch(object, callback));
                };
                this.destroyWatchers = function(){
                    for(var i=0; i<that.watchers.length; i++) {
                        that.watchers[i]();
                    }
                };

                this.clearErrors = function(key) {
                    that.setData('$errors', {}, key);
                };

                this.setErrors = function(key, data, message) {
                    for (var k in data) {
                        if (angular.isArray(data[k])) {
                            data[k] = data[k][0];
                        }
                    }
                    that.setData('$errors', data, key);

                    var isMsg;
                    if ( message === true || message === false) {
                        isMsg = message;
                        message = '提交的数据还存在不足！';
                    } else {
                        isMsg = true;
                    }

                    if (isMsg && message) Landers.response.msgbox({
                        success : false,
                        message : message
                    });

                    return message;
                };

                //定义钩子模块
                // 即将改名： hooks 改名为 hook
                this.hook = this.hooks = {
                    data : {
                        list:{
                            before:[],
                            after:[]
                        },
                        edit:{
                            onload:{
                                before:[],
                                after:[]
                            },
                            submit:{
                                before:[],
                                after:[]
                            }
                        },
                        close:[]
                    },
                    registe: function(key, fns) {
                        var _this = this;
                        if (!angular.isArray(fns)) fns = [fns];
                        angular.forEach(fns, function (fn){
                            if (angular.isFunction(fn)) {
                                Flat.push(_this.data, key, fn);
                            }
                        });
                    },
                    registeOnce: function(key, fn) {
                        if ( Flat.get(this.data, key).length ) return;
                        this.registe( key, fn);
                    },
                    run : function(key, params){
                        var _this = this;
                        var funs = Flat.get(_this.data, key);
                        angular.forEach(funs, function (fn){fn(params);});
                    }
                };

                //创建api与服务端通信对象
                config['api_url'] = config['api_url'] || ('/' + config.key);
                this.api = AngularApi.make(Landers.utils.json.slice(config, [
                    'api_url', 'url_style', 'url_params', 'ajax'
                ]));

                //定义列表操作对象
                // laravel输出分页时，数据位于 data字段中
                this.lister = config.lister || List.make(scope, modelkey + '.$list.data', null);

                //列表操作模块
                this.list = {
                    put:function(data) {
                        that.setData('$list', data);
                    },
                    data:function(data){
                        return that.lister.data(data);
                    },
                    clear:function(){
                        that.lister.clear();
                        that.listitem.selectAll(false);
                        that.setData('$list', {});
                        that.setData('$errors.$list', null);
                        return this;
                    },
                    doShow:function(ajax, opts){
                        opts = opts || {};
                        return ajax.success(function(rspn){
                            if (opts.handleResponse) {
                                opts.handleResponse(rspn);
                            }

                            if (is_api_result(rspn)) {
                                Landers.response.msgbox(rspn);
                                var data = rspn.data;

                                // data 部分是数组 (laravel不分页时)
                                if ( angular.isArray(data) ) {
                                    that.setData('$list', {
                                        data: rspn.data,
                                        page: null
                                    });
                                }

                                // lwap返回格式
                                if ( data.$list ) {
                                    data.$list.data = data.$list.data || [];
                                    data.$list.page = data.$list.page || null;
                                    that.setData('$list', data.$list);
                                }

                                // laravel分页返回格式
                                if ( data.data ) {
                                    that.setData('$list', {
                                        data: data.data,
                                        page: {
                                            current: data.current_page,
                                            last: data.last_page,
                                            size: data.per_page,
                                            total: data.total
                                        }
                                    });
                                }
                            } else {
                                if ( angular.isArray(rspn) ) {
                                    //RESTFUL 返回纯数据
                                    that.setData('$list.data', rspn);
                                }
                            }

                            that.hooks.run('list.after', rspn);
                            opts.onShow && opts.onShow(rspn);
                        }).error(function(rspn){
                            try {
                                that.setData('$errors.$list', rspn.message);
                                that.setData('$list', {data : [], page:{}});
                            } catch (e) {}
                        });
                    },
                    _params:{},
                    show:function(opts){
                        var _this = this;
                        opts = opts || {};
                        that.hooks.run('list.before');

                        var $search = that.getData('$search') || {};
                        for (var k in $search) {
                            if ( k.indexOf('$$$') === 0 ) delete $search.k;
                        }

                        angular.extend(_this._params, opts.params || {});
                        var params = angular.extend({}, _this._params, {
                            search:angular.toJson($search)
                        });
                        var doShow = function() {
                            var ajax = that.api.loading(opts.loading)
                                .list(opts.page, params, opts.pagesize || config.pagesize);
                            return _this.doShow(ajax, opts);
                        };

                        this.clear();
                        if (opts.callbackBefore) {
                            return opts.callbackBefore(doShow);
                        } else {
                            return doShow();
                        }
                    },

                    refresh:function(callback){
                        var opts = {
                            page: that.getData('$list.page.current') || 1,
                            params: {
                                search:angular.toJson(that.getData('$search'))
                            },
                            onShow: callback
                        };
                        this.show(opts);
                    },
                    ifRefresh : function(){
                        if (!that.lister.count()) this.refresh();
                    },
                    preAppend : function(data){
                        return that.lister.preAppend(data);
                    },
                    append : function(data) {
                        return that.lister.append(data);
                    }
                };

                //列表横操作模块
                this.item = this.listitem = {
                    showDeleteConfirm: function($event, item, item_name){
                        var _this = this;
                        Helpers.disable($event);
                        _this.active($event, true);

                        if (item) item.$busy = true;
                        var what_name = item_name ? ( config.name + '【' + item[item_name] + '】') : ('该' + config.name);
                        if (!confirm('确定要删除' + what_name + '吗？')) {
                            _this.active($event, false);
                            Helpers.enable($event);
                            if (item) item.$busy = false;
                            return false;
                        } else {
                            return true;
                        }
                    },
                    show:function(id, params){
                        return that.api.show(id, params)
                            .success(function(rspn){
                                that.setData('$detail', rspn.data);
                            })
                    },
                    data:function($event, key) {
                        return that.lister.item.data($event, key);
                    },
                    swap:function(item1, item2){
                        return that.api.get('swap-sorter/' + item1.id + '/' + item2.id)
                            .success(function(){
                                var index1 = that.lister.mapIndex(item1.id);
                                var index2 = that.lister.mapIndex(item2.id);
                                that.lister.item.swap(index1, index2);
                            })
                            .error(function(rspn){
                                Landers.response.msgbox(rspn);
                            });
                    },
                    moveUp:function(item1) {
                        var index1 = that.lister.mapIndex(item1.id), index2;
                        if (index1 === 0) {
                            return alert('当前项已经处于最前了');
                        } else {
                            index2 = index1 - 1;
                        }
                        var item2 = this.data(index2);
                        this.swap(item1, item2);
                    },
                    moveDown:function(item1){
                        var index1 = that.lister.mapIndex(item1.id), index2;
                        if (index1 == that.list.data().length-1) {
                            return alert('当前项已经处于最后了');
                        } else {
                            index2 = index1 + 1;
                        }
                        var item2 = this.data(index2);
                        this.swap(item1, item2);
                    },
                    count:function(){
                        var data = that.lister.data();
                        return data ? that.lister.data().length : -1;
                    },
                    update:function($event, dat, item){
                        if ( item ) {
                            angular.extend(item, dat);
                        } else {
                            that.lister.item.update($event, dat);
                        }
                    },
                    active:function($event, is_active){
                        return that.lister.item.active($event, is_active);
                    },
                    select:function($event, is_check){
                        return that.lister.item.select($event, is_check);
                    },
                    selectAll : function(bool){
                        var data = that.lister.data();
                        if ( !data ) return false;
                        data = bool ? data.map(function(item){return item.id || item.key;}) : [];
                        that.setData('$items', data, 'selected');
                    },
                    getSelectedId : function(alert_message){
                        if (alert_message === true) {
                            alert_message = '未选择目标数据！';
                        }
                        var ids = that.getData('$items', 'selected');
                        if ((!ids || !ids.length) && alert_message) {
                            alert(alert_message);
                            return null;
                        }
                        return ids;
                    },
                    remove:function(ids, lister) {
                        var _this = this;
                        if (!angular.isArray(ids)) {
                            ids = [ids];
                        }
                        var lister = lister || that.lister;
                        angular.forEach(ids, function(id){
                            var index = lister.mapIndex(id);
                            lister.item.remove(index);
                        });

                        var total = that.getData('$list.page.total');
                        if (total) {
                            that.setData('$list.page.total', total - ids.length);
                        }
                    },
                    deleteById:function(ids, lister){
                        var _this = this;
                        return that.api.delete(ids)
                            .success(function(rspn){
                                _this.remove(ids, lister);
                                that.list.ifRefresh();
                            });
                    },
                    // 被 dev_ssetting 重写 deleteItem 调用时，需要传入lister
                    delete:function(item, lister){
                        var _this = this;
                        return _this.deleteById(item.id, lister);
                    },
                    deleteSelected:function(callback){
                        var ids = this.getSelectedId(true);
                        if (!ids) return;
                        if ( !confirm('确定要删除选择的数据？')) return;
                        return this.deleteById(ids)
                            .success(function(rspn){
                                that.setData('$items', [], 'selected');
                                callback && callback(rspn);
                            });
                    }
                };

                this.addedit = {
                    doEdit: function(item, onLoad, params, hook){
                        var _this = this;
                        return that.api.edit(item.id, params || {})
                           .success(function(rspn){
                                var editData = (function(){
                                    if ( !rspn.data ) return rspn;
                                    if ( rspn.data.$edit ) return rspn.data.$edit;
                                    return rspn.data;
                                })();
                                if (editData) that.setData('$edit', editData);
                                onLoad && onLoad(rspn);
                                hook && hook(rspn);
                           });

                    },
                    doUpdate: function($event, item, onSubmit, hook){
                        var data = that.getData('$edit');
                        var _this = this;
                        return that.api.update(item.id, data)
                            .success(function(rspn){
                                var updated_data = null;

                                if ( is_api_result(rspn) ) {
                                    if ( rspn.data ) {
                                        updated_data = rspn.data.$modify || rspn.data.$update || rspn.data;
                                    }
                                } else {
                                    if ( angular.isObject(rspn) ) {
                                        updated_data = rspn;
                                    } else if (rspn === true) {
                                        updated_data = that.getData('$edit');
                                    }
                                }

                                if ( !updated_data ) {
                                    return that.list.refresh();
                                } else {
                                    that.listitem.update($event, updated_data, item);

                                    // 调用回调
                                    onSubmit && onSubmit(rspn);

                                    // 更新到模型缓存
                                    that.setData('$update', updated_data)

                                    // 调用钩子
                                    hook && hook(rspn);
                                }
                            });
                    },
                    doCreate: function(onSubmit, hook){
                        var data = that.getData('$edit');
                        var _this = this;
                        return that.api.create(data)
                            .success(function(rspn){
                                var created_data = null;

                                var page_info = that.getData('$list.page') || {};
                                if ( !('$total' in  page_info) ) {
                                    if ( is_api_result(rspn) ) {
                                        if ( rspn.data ) {
                                            created_data = rspn.data.$create || rspn.data;
                                        }
                                    } else {
                                        if ( angular.isObject(rspn) ) {
                                            created_data = rspn;
                                        } else if (rspn === true) {
                                            created_data = that.getData('$edit');
                                        }
                                    }
                                }

                                if ( !created_data ) {
                                    return that.list.refresh();
                                } else {
                                    that.list.preAppend(created_data);

                                    // 调用回调
                                    onSubmit && onSubmit(rspn);

                                    // 更新到模型缓存
                                    that.setData('$create', created_data)

                                    // 调用钩子
                                    hook && hook(rspn);
                                }
                            });
                    },
                    show: function($event, item, callback){
                        var action = item ?
                            {key : 'edit', text : '编辑', is_edit : true, datId: item.id, event:$event} :
                            {key : 'create', text : '添加', is_edit : false, datId:null, event:$event };

                        var _this = this;

                        var doLoad = function(onLoad, params){
                            that.setData('$edit', {});
                            that.setData('$errors.$edit', null);
                            that.hooks.run('edit.onload.before', {action:action});
                            that.setData('$runtime.is_edit', action.is_edit);
                            if (action.is_edit) {
                                return _this.doEdit(item, onLoad, params, function(rspn){
                                    that.hooks.run('edit.onload.after', {action:action, rspn:rspn});
                                });
                            } else {
                                onLoad && onLoad({});
                                that.hooks.run('edit.onload.after', {action:action});
                            }
                        };
                        var doSubmit = function(onSubmit){
                            that.hooks.run('edit.submit.before', {action:action});
                            return  action.is_edit ?
                                    _this.doUpdate($event, item, onSubmit, function(rspn){
                                        that.hooks.run('edit.submit.after', {action:action, rspn:rspn});
                                    }) :
                                    _this.doCreate( onSubmit, function(rspn){
                                        that.hooks.run('edit.submit.after', {action:action, rspn:rspn});
                                    });
                        };

                        callback(action, doLoad, doSubmit);
                    }
                };


                //新初始化$event, （注：必须不能用angular.extend来扩展，因为scope可能某子controller并继承了父controller）
                that.extendData('$event', {
                    selectAll : function($event) {
                        $event = angular.element($event.target);
                        var checked = $event.prop('checked');
                        that.listitem.selectAll(checked);
                    },
                    refreshList : function( callback ){
                        that.list.refresh( callback );
                    },
                    deleteItem : function($event, item, item_name) {
                        if ( !that.listitem.showDeleteConfirm($event, item, item_name) ) {
                            return {
                                success:function(){
                                    //donoting
                                    return this;
                                },
                                error:function(callback){
                                    callback();
                                    return this;
                                },
                                finally : function(callback) {
                                    callback();
                                    return this;
                                }
                            };
                        } else {
                            return that.listitem.delete(item)
                                .success(function(rspn){
                                    Landers.response.msgbox(rspn);
                                })
                                .error(function(rspn){
                                    Landers.response.msgbox(rspn);
                                    if (item) item.$busy = false;
                                    Helpers.enable($event);
                                })
                                .finally(function(){
                                    that.listitem.active($event, false);
                                });
                        }
                    },
                    deleteSelected : function() {
                        that.listitem.deleteSelected();
                    },
                    search : function(page){
                        var opts = {page : page};
                        var dat = that.getData('$search') || {};
                        if (dat._keyword_ || dat._keyword_by_) {
                            if (!dat._keyword_) {
                                return alert('未输入搜索关键词');
                            }
                            if (!dat._keyword_by_) {
                                return alert('未选择搜索关键词依据');
                            }
                        }
                        var ajax = that.list.show(opts);
                        if ( this.onSearch ) this.onSearch(ajax);
                    },
                    showPage : function(page){
                        var ajax = this.search(page);
                        if ( this.onShowPage ) this.onShowPage(ajax);
                    },
                    resetSearch : function(){
                        that.setData('$search', {});
                        scope.$emit('reset-search');
                        scope.$broadcast('reset-search');
                        that.list.show();
                    },
                    moveUp : function($event, item) {
                        that.listitem.moveUp(item);
                    },
                    moveDown : function($event, item) {
                        that.listitem.moveDown(item);
                    },
                    opposite : function($event, item, prop){
                        return that.api.opposite(item.id, prop)
                            .success(function(rspn){
                                console.log()
                                if ( angular.isObject(rspn.data) ) {
                                    item[prop] = rspn.data[prop];
                                }

                                if (
                                    rspn.data === true ||
                                    rspn.data === 1 ||
                                    rspn.data === false ||
                                    rspn.data === 0
                                ) {
                                    item[prop] = rspn.data;
                                }
                            });
                    }
                });

                that.extendData('$enums', $rootScope.$enums || $rootScope.Enums || {});

                scope.$on('$destroy', function(){
                    for (var i=0; i<watchers.length; i++) watches[i]();

                    Landers.response.hide();
                });
            }

            return {
                make:function(scope, config){
                    return new Model(scope, config);
                },
                isApiResult: is_api_result,
            };
        }];
    });