;angular.module('Landers.angular')
    // 建立web socket连接，对象存在rootScope下，方便全局调用
    // 监听web socket发来的消息，由rootScope进行广播
    .provider('GWebSocket', function(){
        var provider = this;
        this.wsUrl = null;
        this.showLog = false;
        this.onOpen = null;

        // 初始化超时时间
        var initialTimeout = 1500;

        // 设置是否断开自动重连
        var reconnectIfNotNormalClose = true;

        this.$get = ['$rootScope','$websocket', function($rootScope, $websocket) {
            if (!provider.wsUrl) return;

            var ws = $rootScope.ws, authed = false;

            if (!ws) {
                var ws_protocol = window.location.protocol.substring(0,5) == 'https' ? 'wss' : 'ws';
                var ws_url = ws_protocol + '://' + provider.wsUrl;
                $rootScope.ws = ws = $websocket(ws_url);
            }

            // 设置断开自动重连
            ws.reconnectIfNotNormalClose = reconnectIfNotNormalClose;

            // 设置初始化连接超时时间
            ws.initialTimeout = initialTimeout;

            // 设置创建连接时的回调函数，发送认证信息到服务器
            ws.onOpen(function () {
                provider.onOpen && provider.onOpen();
            });

            // 设置回调函数，收到消息后向全局广播
            ws.onMessage(function (msg) {
                if (msg && msg.data) {
                    var dat = angular.fromJson(msg.data);
                    var cmd = dat.cmd;
                    var data = dat.data;
                    show_log('receive message：', cmd, data);

                    // 广播至各个scope
                    $rootScope.$broadcast(cmd, data);
                }
            });

            ws.onClose(function() {
                show_log('websocket is closed', '', '');
            });

            var watcher = {
                key: 'websocket_listers',
                get: function(scope, cmd) {
                    scope[this.key] = scope[this.key] || {};
                    return scope[this.key][cmd];
                },
                add: function(scope, cmd, on_result) {
                    scope[this.key] = scope[this.key] || {};
                    scope[this.key][cmd] = on_result;
                },
                remove: function(scope, cmd) {
                    scope[this.key] = scope[this.key] || {};
                    delete scope[this.key][cmd];
                }
            };

            var send = function(scope, cmd, data, lisener){
                //cmd:命令格式统一为类名_方法名，例如user_login
                //data:json对象包
                var pack = {"cmd" : cmd, "data": data};
                if ( lisener && !watcher.get(scope, cmd)) {
                    var ret = scope.$on(cmd, lisener);
                    watcher.add(scope, cmd, ret);
                }
                return ws.send(angular.toJson(pack));
            };

            var show_log = function(msg, cmd, data) {
                if (provider.showLog) {
                    console.log(msg, cmd, data);
                }
            };

            return {
                // 发送请求到服务端
                send: function(scope, cmd, data, lisener){
                    if (!scope) return;
                    if ( authed ) {
                        send(scope, cmd, data, lisener)
                            .then(function () {
                                show_log('success to send message:', cmd, data);
                            }, function () {
                                // 发送失败的消息会被push到ws对象的sendQueue数组中
                                show_log('failed to send message:', cmd, data);
                            });
                    } else {
                        var self = arguments.callee;
                        setTimeout(function(){
                            self(cmd, data);
                        }, 300);
                    }
                },

                cancel: function(scope, cmd){
                    var destroyWatch = watcher.get(scope, cmd);
                    if ( destroyWatch ) {
                        destroyWatch();
                        watcher.remove(scope, cmd);
                    }
                },

                auth : function(ws_token) {
                    if ( !provider.onOpen ) {
                        provider.onOpen = function() {
                            send($rootScope, 'user_auth', {token: ws_token}, function(event, response){
                                authed = response.success;
                            });
                        };
                    }

                    $rootScope.$on('user_need_login', provider.onOpen)

                    provider.onOpen();
                }
            };
        }];
    });