if (window.$) {
    $.fn.attachWatcher = function(fn) {
        $(this).data('angularDestroyWatch', fn);
    };
    $.fn.destroyWatch = function(){
        var fn = $(this).data('angularDestroyWatch');
        if (fn) fn();
        return this;
    };
}
angular.module('Landers.utils', [])
    .factory('Helpers', ['Flat', function(Flat){
        return {
            // seekScopeByDeep:function(scope, deep) {
            //     deep = parseInt(deep);
            //     var myScope = scope;
            //     if (!deep) return myScope;
            //     for (var i = 1; i <= deep; i++) {
            //         if (myScope.$parent) {
            //             myScope = myScope.$parent;
            //         } else {
            //             break;
            //         }
            //     }
            //     return myScope;
            // },

            seekScope : function(scope, parentId) {
                var myScope = scope;
                if ( !parentId) return myScope;
                while (myScope.$id != parentId) {
                    if (myScope.$parent) {
                        myScope = myScope.$parent;
                    } else {
                        break;
                    }
                }
                return myScope;
            },
            toJqEle : function($event) {
                if (!$event) return $();
                var obj = angular.isElement($event) ? $event : $event.target;
                return angular.element(obj);
            },
            disable : function($event){
                $event = this.toJqEle($event);
                return Landers.helpers.disable($event);
            },
            enable : function($event) {
                $event = this.toJqEle($event);
                return Landers.helpers.enable($event);
            },
            // showFormErrors : function(scope, key, rspn) {
            //     var errors = rspn.errors;
            //     if (errors) {
            //         for (var k in errors) {
            //             var datakey = ['$errors', key, k].join('.');
            //             Flat.set(scope, datakey, errors[k][0]);
            //         }
            //     }
            //     rspn.message = rspn.message || '提交的数据还存在不足！';
            //     Landers.response.msgbox(rspn);
            // },
            // clearFormErrors : function(scope, key) {
            //     Flat.set(scope, key + '.errors', {} );
            // },
            domainUrl : function(domainUrl, url) {
                if (!domainUrl) return url;
                var len = domainUrl.length;
                var pre = Landers.utils.str.left(url, len);
                if (pre === domainUrl) return url;
                else return domainUrl + url;
            }
        };
    }])
    .factory('PromiseLoads', ['$q', function($q) {
        return {
            asyn:function(tasks, callback){
                var promises = [];
                angular.forEach(tasks, function(task){
                    var deferred = $q.defer();
                    var ajax = task(deferred);
                    if (ajax) ajax.success(function(){
                        deferred.resolve();
                    }).error(function(){
                        deferred.reject();
                    });
                    promises.push(deferred.promise);
                });
                $q.all(promises).then(function(){
                    // 曾在 initCreate() 调用无延时出过错
                    setTimeout(callback, 200);
                });
            }
        };
    }])
    .factory('HttpInterceptor', ['$log', '$q', '$location', function($log, $q, $location) {
        return {
            response: function(response) {
                var deferred = $q.defer();
                deferred.resolve(response);
                return deferred.promise;
            },

            responseError : function(rejection){
                var deferred = $q.defer();
                deferred.reject(rejection);
                return deferred.promise;
            }
        };
    }])
    .factory('DataConvert', function(){
        return {
            checkbox:function(xvar) {
                var ret;
                if (angular.isString(xvar)) {
                    xvar = angular.toJson(xvar);
                }
                if (angular.isArray(xvar)) {
                    ret = {};
                    angular.forEach(xvar, function(item){
                        ret[item] = true;
                    });
                } else {
                    if (angular.isObject(xvar)) {
                        ret = [];
                        for (var k in xvar) {
                            if (xvar[k]) ret.push(k);
                        }
                    }
                }
                return ret;
            }
        };
    })
    .factory('InjectService', ['$ocLazyLoad', '$injector', function($ocLazyLoad, $injector){
        var injected = {};
        return {
            urls:function(urls, callback){
                $ocLazyLoad.load(urls).then(callback);
            },
            module : function(url) { //url, inject, callback
                var injects = [], callback;
                for (var i=1; i<arguments.length;i++) {
                    if (angular.isFunction(arguments[i])) {
                        callback = arguments[i];
                    } else {
                        injects.push(arguments[i]);
                    }
                }
                var strInjects = injects.join('.');
                if (injected[strInjects]) {
                    callback && callback(injected[strInjects]);
                } else {
                    this.urls(url, function(){
                        if (callback) injects.push(callback);
                        $injector.invoke(injects);
                    });
                }
            }
        };
    }])
    .factory('BaseService', ['Ajax', 'Runtime', function (Ajax, Runtime){
        function BaseService(scope, extend, events){
            this.ajax = Ajax.make();
            this.runtime = Runtime.make(scope);
            angular.extend(this, extend || {});
            angular.extend(scope.$events, events || {});
        }
        return {
            make:function(scope, extend, events) {
                return new BaseService(scope, extend, events);
            }
        };
    }])
    .filter('round', function() {
        return function(num, n){return Math.round(num*Math.pow(10,n || 0))/Math.pow(10,n || 0); };
    })
    .filter('nl2br', function(){
        return function(str) {
            var s1 = '\n'; s2 = '<br/>';
            s1 = s1.replace(/\{/,'\\{').replace(/\}/,'\\}');
            var re = new RegExp(s1,"gi");
            return str.replace(re,s2);
        };
    })
    .filter('nl2li', function(){
        return function(str) {
            return str.split('\n').map(function(line){
                return '<li>' + line + '</li>';
            }).join('');
        };
    })
    .filter('datetime', function(){
        return function(str, what) {
            var a = str.split(' ');
            switch (what) {
                case 'date' : return a[0];
                case 'time' : return a[1];
                default : return str;
            }
        };
    })
    .factory('Polling', ['$timeout', function($timeout){
        function AjaxPolling(ajaxBuilder, delay){
            var callbacks = [];
            var done_count = 0;
            var _stop = true;

            this.push = function(callback){
                callbacks.push(callback);
                this.start();
            };

            var _callback = function(rspn){
                angular.forEach(callbacks, function(callback){
                    callback(rspn);
                });
            };
            this.execute = function(){
                done_count = 0;
                ajaxBuilder().success(_callback).error(_callback);
            };

            this.done = function(){
                var me = this; done_count++;
                if (done_count == callbacks.length) {
                    $timeout(function(){
                        if (!_stop) me.execute();
                    }, delay || 1000);
                }
            };

            this.start = function(){
                if (!_stop) return;
                _stop = false;
                this.execute();
                return this;
            };

            this.stop = function(){
                _stop = true;
                return this;
            };
        }

        return {
            make:function(ajaxBuilder, delay){
                return new AjaxPolling(ajaxBuilder, delay);
            }
        };
    }])
    .directive('toggleClasses', function() {
        return {
            restrict : 'A',
            require : '?ngModel',
            link : function($scope, $ele, $attrs) {
                var arr1 = $attrs['toggleClasses'].split(',');
                var arr2 = arr1[0].split('|'), callback = arr1[1];
                $ele.data('ToggleClassIndex', 0)
                .addClass(arr2[0]).click(function(){
                    var last_index = $ele.data('ToggleClassIndex');
                    var index = last_index + 1;
                    if (index >= arr2.length) index = 0;
                    for (var i=0; i<arr2.length; i++) $ele.removeClass(arr2[i]);
                    $ele.data('ToggleClassIndex', index).addClass(arr2[index]);
                    callback && $scope[callback](index, arr2[index]);
                });
            }
        };
    })

    .directive('keyToText', ['$timeout', 'Flat', function($timeout, Flat) {
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs) {
                var arr = $attrs['keyToText'].split('|');
                var enums = Flat.get($scope, arr[0]);
                var monitorKey = arr[1];
                var textField = arr[2] ? arr[2] : 'text';
                var listener = function(newValue) {
                    var data = enums[newValue];
                    if (!data) return;

                    var html = data[textField] || data.text;
                    var $text = $ele.find('.key-to-text');
                    if ($text.length) $text.html(html);
                    else $ele.html(html);

                    if (!data._class) return;
                    for (var k in enums) $ele.removeClass(enums[k]['_class']);
                    $ele.addClass(data._class);
                };
                var monitorVal = monitorKey.match(/\[(.*)\]/);
                if (monitorVal) {
                    $scope.$watch( function(){return monitorVal[1];}, listener);
                } else {
                    $scope.$watch( monitorKey, listener);
                }
            }
        };
    }])

    .factory('Location', ['$location', function($location){
        return {
            param:function(index, noNull){
                var url = $location.url().split('/');
                if (index == 'last') index = url.length - 1;
                var ret = url[index];
                if (!ret && noNull !== false) {
                    ret = url[index-1];
                }
                return ret;
            },
            params:function(keys){
                var url = $location.url().split('/');
                var ret = {};
                angular.forEach(keys, function(item, i){
                    ret[item] = url[i+2];
                });
                return ret;
            }
        };
    }])

    // TabIndex
    .factory('TabIndex', ['$state', 'Location', function ($state, Location){
        function TabIndex(scope) {
            this.show = function(callbacks){
                var that = this;
                var stateData = $state.$current.data || {};
                var tab_index = typeof stateData.tabIndex !== 'undefined' ?
                                stateData.tabIndex : Location.param(2);
                scope.tabIndex = tab_index;
                var _callback = callbacks[tab_index] || callbacks['default'];
                if (_callback) _callback(tab_index);
                else throw('callbacks[' + tab_index + ']未定义');
            };

        }
        return {
            make:function(_scope){return new TabIndex(_scope);},
            get:function(){return Location.param(2);}
        };
    }])

    // .provider()

    // Ajax
    .provider('Ajax', function() {
        var provider = this;
        this.hosturl = '';
        this.$get = ['$http', '$rootScope', 'Form', '$q', function($http, $rootScope, Form, $q){
            var csrf_token;
            function Ajax(opts) {
                var _debug, _loading;
                var _hosturl = provider.hosturl;
                var cancellers = [];
                var prevents = {
                    error : false,
                    error_once : false
                };
                var build_url = function(url, params) {
                    if (params) {
                        url = Landers.url.qs.set(url, params);
                    }
                    return _hosturl + url;
                };
                this.setHosturl = function(hosturl){
                    _hosturl = hosturl;
                    return this;
                }
                this.debug = function(is){
                    _debug = is !== false && is !== 0;
                    return this;
                };
                this.loading = function(is){
                    _loading = is === true || is === undefined;
                    return this;
                };
                this._get = function(method, url, params){
                    var that = this;
                    url = build_url(url, params);
                    if (_debug) {
                        return Landers.ajax.debug().get(url);
                    } else {
                        var ret, config = {ignoreLoadingBar: !_loading};
                        var canceler = $q.defer();
                        cancellers.push(canceler);
                        // 添加timeout设置为promise对象，才可以取消未完成的request请求
                        config.timeout = canceler.promise;
                        switch (method.toLowerCase()) {
                            case 'get' :
                                ret = $http.get(url, config); break;
                            case 'delete':
                                ret = $http.delete(url, config); break;
                        }
                        ret.success(that.success).error(that.error)
                           .finally(function(){cancellers.pop();});
                        _loading = _debug = false;
                        return ret;
                    }
                };
                this.get = function(url, params) {
                    return this._get('get', url, params);
                };
                this.delete = function(url, params){
                    return this._get('delete', url, params);
                };
                this._post = function(method, url, xvar, params){
                    var that = this;
                    url = build_url(url, params);
                    // var data = angular.isString(xvar) || angular.isElement(xvar) ? Form.data(xvar) : xvar;
                    var data = xvar || {};

                    if (csrf_token) data._token = csrf_token;
                    if (_debug) {
                        return Landers.ajax.debug().post( url, data );
                    } else {
                        var ret, config = {ignoreLoadingBar: !_loading}; //, withCredentials:true
                        var canceler = $q.defer();
                        cancellers.push(canceler);
                        // 添加timeout设置为promise对象，才可以取消未完成的request请求
                        config.timeout = canceler.promise;
                        switch (method.toLowerCase()) {
                            case 'post' :
                                ret = $http.post(url, data, config); break;
                            case 'put' :
                                data['_method'] = 'put';
                                ret = $http.put(url, data, config); break;
                        }
                        ret.success(that.success).error(that.error)
                           .finally(function(){cancellers.pop();});

                        _loading = _debug = false;
                        return ret;
                    }
                };
                this.post = function(url, xvar, params){
                    return this._post('post', url, xvar, params);
                };
                this.put = function(url, data, params){
                    return this._post('put', url, data, params);
                };
                this.success = function(rspn, code, header){
                    csrf_token = rspn.csrf_token;
                    if (rspn.code == 1 && rspn.message) {
                        Landers.msgbox(rspn);
                    }
                };
                this.error = function(rspn) {
                    if (angular.isObject(rspn)) {
                        csrf_token = rspn.csrf_token || csrf_token;
                    } else {
                        console.log('无响应404');
                        rspn = {
                            success : false,
                            message : '' //'远程无响应或OPTION方法',
                        };
                    }
                };
                this.cancelAll = function(i){
                    if ( cancellers.length ) {
                        var canceller;
                        while (canceller = cancellers.pop()) {
                            canceller.resolve("user cancelled");
                        }
                    }
                };
            }
            return {
                make:function(opts){
                    return new Ajax(opts);
                },
                csrf_token:function(){
                    return csrf_token;
                }
            };
        }];
    })

    // Dialog
    .provider('Dialog', function(){
        var provider = this;
        this.skin = 'ulan';
        this.defaults = {
            effect : 'random'
        };
        this.$get = ['$compile', '$timeout', function($compile, $timeout) {

            //先异步加载css文件，以便Landers.dialog.show为非异步
            Landers.dialog.loadmod(provider.skin);
            return {
                confirm:function(scope, opts, callbacks){
                    angular.extend(opts, {
                        top : '30%',
                        yes: {text:'　确定　', fn:callbacks.yesFn },
                        no: {text:'　取消　', fn:callbacks.noFn },
                        angular : {compile:$compile, scope:scope, timeout:$timeout}
                    });
                    return Landers.dialog.show(opts);
                },
                show:function(scope, opts){
                    var extOpts = opts.compile === false ? {} : {
                        angular : {compile:$compile, scope:scope, timeout:$timeout},
                    };

                    opts = angular.extend({}, provider.defaults, extOpts, opts);
                    return Landers.dialog.show(opts, provider.skin);
                }
            };
        }];
    })

    .factory('Flat', function () {
        function Flat(scope, J){
            var that = Flat;
            this.apply = function(){
                if (!scope.$$phase) scope.$apply();
                return this;
            };
            this.get = function(key, subkey) {
                return that.get(J, key, subkey);
            };
            this.pull = function(key) {
                var ret = that.pull(J, key);
                this.apply();
                return ret;
            };
            this.set = function(key, value, subkey){
                that.set(J, key, value, subkey);
                return this.apply();
            };
            this.setOnce = function(key, value, subkey){
                var ret = that.setOnce(J, key, value, subkey);
                this.apply();
                return ret;
            };
            this.delete = function(key) {
                that.delete(J, key);
                return this.apply();
            };
            this.clear = function(){
                that.clear(J);
                return this.apply();
            };
            this.extend = function(json, subkey){
                that.extend(J, json, subkey);
                return this.apply();
            };
            this.cut = function(key, index) {
                that.cut(J, key, index);
                return this.apply();
            };
            this.copy = function(key, start, end) {
                that.copy(J, key, start, end);
                return this.apply();
            };
            this.shift = function(key) {
                that.shift(J, key);
                return this.apply();
            };
            this.unshift = function(key, value) {
                that.unshift(J, key, value);
                return this.apply();
            };
            this.push = function(key, value){
                that.push(J, key, value);
                return this.apply();
            };
            this.pop = function(key) {
                that.pop(J, key);
                return this.apply();
            };
        }
        angular.extend(Flat, {
            make : function(scope, J) {
                return new Flat(scope, J);
            },
            parse : function(J, key, subkey){
                if (subkey) key += '.' + subkey;
                var a = key.split('.');
                J = J || {};
                if (a.length == 1) {
                    return {
                        obj:J, key:key
                    };
                } else {
                    for (var i=0; i<a.length-1; i++) {
                        J = J || {};
                        if ( ! (a[i] in J) ) {
                            J[a[i]] = {};
                        }
                        J[a[i]] = J[a[i]] || {};
                        J = J[a[i]];
                    }
                    return {
                        obj:J,
                        key:a[i]
                    };
                }
            },
            get : function(J, key, subkey) {
                var parse = this.parse(J, key, subkey);
                return parse.obj[parse.key];
            },
            pull : function(J, key){
                var ret = this.get(J, key);
                this.delete(J, key);
                return ret;
            },
            set : function(J, key, value, subkey){
                var parse = this.parse(J, key, subkey);
                // parse.obj = parse.obj || {}; //promote 为空时
                parse.obj[parse.key] = value;
                return this;
            },
            setOnce : function(J, key, value, subkey){
                var ret = this.get(J, key, subkey);
                if ( ret ) return ret;
                this.set(J, key, value, subkey);
                return value;
            },
            delete : function(J, key) {
                var parse = this.parse(J, key);
                delete parse.obj[parse.key];
                return this;
            },
            clear : function(J) {
                for (var k in J) delete J[k];
                J = {};
                return this;
            },
            extend : function(J, jdat, subkey) {
                if (subkey) {
                    var value = this.get(J, subkey) || {};
                    angular.extend(value, jdat);
                    this.set(J, subkey, value);
                } else {
                    for (var key in jdat) {
                        this.set(J, key, jdat[key]);
                    }
                }
                return this;
            },
            cut : function(J, key, index) {
                var parse = this.parse(J, key);
                parse.obj[parse.key].splice(index, 1);
                return parse.obj[parse.key];
            },
            copy : function(J, key, start, end) {
                var parse = this.parse(J, key);
                return parse.obj[parse.key].slice(start);
            },
            shift : function(J, key) {
                var parse = this.parse(J, key);
                parse.obj[parse.key].shift();
                return this;
            },
            unshift : function(J, key, value) {
                var parse = this.parse(J, key);
                try {
                    parse.obj[parse.key].unshift(value);
                }catch(e){
                    console.log(J);
                    console.log(key);
                    console.log(value);
                }
                return this;
            },
            push : function(J, key, value){
                var parse = this.parse(J, key);
                parse.obj[parse.key] = parse.obj[parse.key] || [];
                parse.obj[parse.key].push(value);
                return this;
            },
            pop : function(J, key) {
                var parse = this.parse(J, key);
                parse.obj[parse.key].pop();
                return this;
            }
        });

        return Flat;
    })

    .factory('Runtime', ['Flat', function (Flat) {
        var cache = {};
        function runtime (scope){
            if ( !cache[scope.$id] ) {
                scope.$runtime = {};
                cache[scope.$id] = true;
            }
            this.__proto__ = Flat.make(scope, scope.$runtime);
        }
        return {
            make:function(scope){
                return new runtime(scope);
            }
        };
    }])
    .factory('Listitem', ['Flat', function (Flat){
        var _class = {
            list:'list',
            item:'list-item',
            checked:'checked',
            actived:'active',
            head:'list-head',
            checker:'checker'
        };
        var data = function(scope, datakey, index, data_or_key){
            if (typeof data_or_key === 'undefined') {
                return Flat.get(scope, [datakey, index].join('.'));
            } else {
                if (angular.isString(data_or_key)) {
                    var dat = Flat.get(scope, [datakey, index].join('.')) || {};
                    return dat[data_or_key];
                } else {
                    Flat.set(scope, [datakey, index].join('.'), data);
                }
            }
        };
        var update = function(scope, datakey, index, data, field) {
            var _update = function(_field, _data){
                var _datakey = [datakey, index, _field];
                Flat.set(scope, _datakey.join('.'), _data);
            };
            if (field) {
                _update(field, data);
            } else {
                for (var fieldKey in data){
                    _update(fieldKey, data[fieldKey]);
                }
            }
            if (!scope.$$phase) scope.$apply();
        };
        var remove = function(scope, datakey, index){
            if ( index >= 0) {
                var data = Flat.get(scope, datakey);
                data.splice(index, 1);
                if (!scope.$$phase) scope.$apply();
            }
        };
        function listitem(lister, $event) {
            var $item = (function(){
                if (angular.isElement($event)) {
                    return $event;
                } else {
                    $event = angular.element($event.target);
                    return $event.closest('.' + _class.item);
                }
            })();
            this.get = function(){
                return $item;
            };
            this.getlist = function() {
                return $item.closest('.' + _class.list);
            };
            this.index = function(){
                var $head = $item.siblings('.' + _class.head);
                var index = $item.index();
                return $head.length ? index - 1 : index;
            };
            this.data = function(data_or_key) {
                var index = this.index();
                return data(lister.scope, lister.datakey, index, data_or_key);
            };
            this.update = function(data, field){
                var index = this.index();
                update(lister.scope, lister.datakey, index, data, field);
            };
            this.remove = function() {
                var index = this.index();
                remove(lister.scope, lister.datakey, index);
            };
            this.toggleClass = function(is, classname){
                if (is) {
                    $item.addClass(classname);
                } else {
                    $item.removeClass(classname);
                }
                return this;
            };
            this.active = function(is_active){
                return this.toggleClass(is_active, _class.actived);
            };
            this.select = function(is_check){
                return this.toggleClass(is_check, _class.checked);
            };
        }
        return {
            make:function(scope, $event) {
                return new listitem(scope, $event);
            },
            _class:_class,
            data:data,
            update:update,
            remove:remove
        };
    }])
    .factory('List', ['Flat', 'Listitem', function (Flat, Listitem) {
        var _class = {
            list:'list',
            item:'list-item',
            checked:'checked',
            actived:'active',
            head:'list-head',
            checker:'checker'
        };
        function list(scope, datakey, xExtends) {
            var that = this;
            angular.extend(this, xExtends);
            this.scope = scope;
            this.datakey = datakey;
            var _apply = function(){
                if (!scope.$$phase) scope.$apply();
            };
            this.preAppend = function(data){
                Flat.unshift(scope, datakey, data); _apply();
                return this;
            };
            this.append = function(data) {
                Flat.push(scope, datakey, data); _apply();
                return this;
            };
            this.push = function(data) {
                return this.append(data);
            };
            this.pop = function() {
                Flat.pop(scope, datakey); _apply();
                return this;
            };
            this.count = function(){
                var data = Flat.get(scope, datakey);
                return data ? data.length : 0;
            };
            this.data = function(data) {
                if (data === undefined) {
                    return Flat.get(scope, datakey);
                } else {
                    if (angular.isString(data)) {
                        data = angular.fromJson(data);
                    }
                    data = data || [];
                    Flat.set(scope, datakey, data); _apply();
                    return that;
                }
            };
            this.setLength = function(length) {
                Flat.cut(scope, datakey, length);
            };
            this.mapIndex = function(ids){
                var _list = {};
                var data = that.data();
                if (!data) return -1;
                data.forEach(function(item, index){
                    _list[item.id] = index;
                });
                if (angular.isArray(ids)) {
                    var ret = [];
                    angular.forEach(ids, function(id){
                        ret.push(_list[id]);
                    });
                    return ret;
                } else {
                    return _list[ids];
                }
            };
            this.mapId = function(index){
                var _list = {};
                return this.data(index, 'id');
            };
            this.clear = function(){
                Flat.set(scope, datakey, []); _apply();
            };
            this.item = {
                make:function($event) {
                    return Listitem.make(that, $event);
                },
                get:function($event){
                    return this.make($event).get();
                },
                getlist:function($xvar) {
                    return this.make($event).getlist();
                },
                swap:function(index1, index2){
                    var data1 = this.data(index1);
                    var data2 = this.data(index2);
                    var data = that.data();
                    data[index1] = data2;
                    data[index2] = data1;
                    return data;
                },
                index:function($xvar){
                    if (!angular.isNumber($xvar)) {
                        return this.make($xvar).index();
                    } else {
                        return $xvar;
                    }
                },
                data:function($xvar, data_or_key) {
                    var index = this.index($xvar);
                     return Listitem.data(scope, datakey, index, data_or_key);
                },
                update:function($xvar, data, field){
                    var index = this.index($xvar);
                    Listitem.update(scope, datakey, index, data, field);
                    return this;
                },
                remove:function($xvar) {
                    var index = this.index($xvar);
                    Listitem.remove(scope, datakey, index);
                    return that;
                },
                active:function($event, is_active){
                    return this.make($event).active(is_active);
                },
                select:function($event, is_select){
                    return this.make($event).select(is_select);
                }
            };
        }
        return {
            make:function(scope, datakey, initData, extendsMethod){
                if (initData === undefined) initData = [];
                if (!Flat.get(scope, datakey) && initData) {
                    Flat.set(scope, datakey, initData);
                }
                return new list(scope, datakey, extendsMethod);
            },
            _class:_class
        };
    }])

    .factory('Form', function () {
        return {
            get_by_event : function($event) {
                var ele = $($event.target);
                return ele.closest('form');
            },

            get_data_by_event : function($event) {
                var form = this.get_by_event($event);
                return Landers.form.data(form);
            },

            get:function(name){
                return angular.element('[name=' + name + ']');
            },

            data : function(form, data) {
                if ('string' == typeof form) {
                    form = this.get(form);
                }
                return Landers.form.data(form);
            }
        };
    })

    // 建立web socket连接，对象存在rootScope下，方便全局调用
    // 监听web socket发来的消息，由rootScope进行广播
    .provider('GWebSocket', function(){
        var provider = this;
        this.wsUrl = null;
        this.showLog = false;
        this.onOpen = null;

        // 初始化超时时间
        var initialTimeout = 1500;

        // 设置是否断开自动重连
        var reconnectIfNotNormalClose = true;

        this.$get = ['$rootScope','$websocket', function($rootScope, $websocket) {
            if (!provider.wsUrl) return;

            var ws = $rootScope.ws, authed = false;

            if (!ws) {
                var ws_protocol = window.location.protocol.substring(0,5) == 'https' ? 'wss' : 'ws';
                var ws_url = ws_protocol + '://' + provider.wsUrl;
                $rootScope.ws = ws = $websocket(ws_url);
            }

            // 设置断开自动重连
            ws.reconnectIfNotNormalClose = reconnectIfNotNormalClose;

            // 设置初始化连接超时时间
            ws.initialTimeout = initialTimeout;

            // 设置创建连接时的回调函数，发送认证信息到服务器
            ws.onOpen(function () {
                provider.onOpen && provider.onOpen();
            });

            // 设置回调函数，收到消息后向全局广播
            ws.onMessage(function (msg) {
                if (msg && msg.data) {
                    var dat = angular.fromJson(msg.data);
                    var cmd = dat.cmd;
                    var data = dat.data;
                    show_log('receive message：', cmd, data);

                    // 广播至各个scope
                    $rootScope.$broadcast(cmd, data);
                }
            });

            ws.onClose(function() {
                show_log('websocket is closed', '', '');
            });

            var watcher = {
                key: 'websocket_listers',
                get: function(scope, cmd) {
                    scope[this.key] = scope[this.key] || {};
                    return scope[this.key][cmd];
                },
                add: function(scope, cmd, on_result) {
                    scope[this.key] = scope[this.key] || {};
                    scope[this.key][cmd] = on_result;
                    console.log(cmd, 'add', scope.$id);
                },
                remove: function(scope, cmd) {
                    scope[this.key] = scope[this.key] || {};
                    delete scope[this.key][cmd];
                    console.log(cmd, 'remove',  scope.$id);
                }
            };

            var send = function(scope, cmd, data, lisener){
                //cmd:命令格式统一为类名_方法名，例如user_login
                //data:json对象包
                var pack = {"cmd" : cmd, "data": data};
                if ( lisener && !watcher.get(scope, cmd)) {
                    var ret = scope.$on(cmd, lisener);
                    watcher.add(scope, cmd, ret);
                }
                return ws.send(angular.toJson(pack));
            };

            var show_log = function(msg, cmd, data) {
                if (provider.showLog) {
                    console.log(msg, cmd, data);
                }
            };

            return {
                // 发送请求到服务端
                send: function(scope, cmd, data, lisener){
                    if (!scope) return;
                    if ( authed ) {
                        send(scope, cmd, data, lisener)
                            .then(function () {
                                show_log('success to send message:', cmd, data);
                            }, function () {
                                // 发送失败的消息会被push到ws对象的sendQueue数组中
                                show_log('failed to send message:', cmd, data);
                            });
                    } else {
                        var self = arguments.callee;
                        setTimeout(function(){
                            self(cmd, data);
                        }, 300);
                    }
                },

                cancel: function(scope, cmd){
                    var destroyWatch = watcher.get(scope, cmd);
                    if ( destroyWatch ) {
                        destroyWatch();
                        watcher.remove(scope, cmd);
                    }
                },

                auth : function(ws_token) {
                    if ( !provider.onOpen ) {
                        provider.onOpen = function() {
                            send($rootScope, 'user_auth', {token: ws_token}, function(event, response){
                                authed = response.success;
                            });
                        };
                    }

                    $rootScope.$on('user_need_login', provider.onOpen)

                    provider.onOpen();
                }
            };
        }];
    });